@@ -7,17 +7,22 @@ The one parameter exponential family distributions used by GLM.
 # for comparison to R, and McCullagh and Nelder
 
 
-import warnings
 import inspect
+import warnings
+
 import numpy as np
-from scipy import special
-from . import links as L
-from . import varfuncs as V
+from scipy import special, stats
+
+from statsmodels.compat.scipy import SP_LT_17
+from statsmodels.tools.sm_exceptions import (
+    ValueWarning,
+    )
+from . import links as L, varfuncs as V
 
 FLOAT_EPS = np.finfo(float).eps
 
 
-class Family(object):
+class Family:
     """
     The parent class for one-parameter exponential families.
 
@@ -29,6 +34,10 @@ class Family(object):
     variance : a variance function
         Measures the variance as a function of the mean probabilities.
         See the individual families for the default variance function.
+    check_link : bool
+        If True (default), then and exception is raised if the link is invalid
+        for the family.
+        If False, then the link is not checked.
 
     See Also
     --------
@@ -57,13 +66,14 @@ class Family(object):
         #  <statsmodels.family.links.Power object at 0x9a4236c>]
         # for Poisson...
         self._link = link
-        if not isinstance(link, L.Link):
-            raise TypeError("The input should be a valid Link object.")
-        if hasattr(self, "links"):
-            validlink = max([isinstance(link, _) for _ in self.links])
-            if not validlink:
-                errmsg = "Invalid link for family, should be in %s. (got %s)"
-                raise ValueError(errmsg % (repr(self.links), link))
+        if self._check_link:
+            if not isinstance(link, L.Link):
+                raise TypeError("The input should be a valid Link object.")
+            if hasattr(self, "links"):
+                validlink = max([isinstance(link, _) for _ in self.links])
+                if not validlink:
+                    msg = "Invalid link for family, should be in %s. (got %s)"
+                    raise ValueError(msg % (repr(self.links), link))
 
     def _getlink(self):
         """
@@ -74,17 +84,16 @@ class Family(object):
     # link property for each family is a pointer to link instance
     link = property(_getlink, _setlink, doc="Link function for family")
 
-    def __init__(self, link, variance):
+    def __init__(self, link, variance, check_link=True):
+        self._check_link = check_link
         if inspect.isclass(link):
-            warnmssg = "Calling Family(..) with a link class as argument "
-            warnmssg += "is deprecated.\n"
-            warnmssg += "Use an instance of a link class instead."
-            lvl = 2 if type(self) is Family else 3
-            warnings.warn(warnmssg,
-                          category=DeprecationWarning, stacklevel=lvl)
-            self.link = link()
-        else:
-            self.link = link
+            warnmssg = (
+                "Calling Family(..) with a link class is not allowed. Use an "
+                "instance of a link class instead."
+            )
+            raise TypeError(warnmssg)
+
+        self.link = link
         self.variance = variance
 
     def starting_mu(self, y):
@@ -378,6 +387,10 @@ class Poisson(Family):
         The default link for the Poisson family is the log link. Available
         links are log, identity, and sqrt. See statsmodels.families.links for
         more information.
+    check_link : bool
+        If True (default), then and exception is raised if the link is invalid
+        for the family.
+        If False, then the link is not checked.
 
     Attributes
     ----------
@@ -392,15 +405,19 @@ class Poisson(Family):
     statsmodels.genmod.families.family.Family : Parent class for all links.
     :ref:`links` : Further details on links.
     """
-    links = [L.log, L.identity, L.sqrt]
+    links = [L.Log, L.Identity, L.Sqrt]
     variance = V.mu
     valid = [0, np.inf]
     safe_links = [L.Log, ]
 
-    def __init__(self, link=None):
+    def __init__(self, link=None, check_link=True):
         if link is None:
-            link = L.log()
-        super(Poisson, self).__init__(link=link, variance=Poisson.variance)
+            link = L.Log()
+        super(Poisson, self).__init__(
+            link=link,
+            variance=Poisson.variance,
+            check_link=check_link
+            )
 
     def _resid_dev(self, endog, mu):
         r"""
@@ -493,6 +510,28 @@ class Poisson(Family):
         resid *= np.sqrt(var_weights)
         return resid
 
+    def get_distribution(self, mu, scale=1., var_weights=1.):
+        r"""
+        Frozen Poisson distribution instance for given parameters
+
+        Parameters
+        ----------
+        mu : ndarray
+            Usually but not always the fitted mean response variable.
+        scale : float
+            The scale parameter is ignored.
+        var_weights : array_like
+            1d array of variance (analytic) weights. The default is 1.
+            var_weights are ignored for Poisson.
+
+        Returns
+        -------
+        distribution instance
+
+        """
+
+        return stats.poisson(mu)
+
 
 class Gaussian(Family):
     """
@@ -504,6 +543,10 @@ class Gaussian(Family):
         The default link for the Gaussian family is the identity link.
         Available links are log, identity, and inverse.
         See statsmodels.genmod.families.links for more information.
+    check_link : bool
+        If True (default), then and exception is raised if the link is invalid
+        for the family.
+        If False, then the link is not checked.
 
     Attributes
     ----------
@@ -519,14 +562,18 @@ class Gaussian(Family):
     :ref:`links` : Further details on links.
     """
 
-    links = [L.log, L.identity, L.inverse_power]
+    links = [L.Log, L.Identity, L.InversePower]
     variance = V.constant
     safe_links = links
 
-    def __init__(self, link=None):
+    def __init__(self, link=None, check_link=True):
         if link is None:
-            link = L.identity()
-        super(Gaussian, self).__init__(link=link, variance=Gaussian.variance)
+            link = L.Identity()
+        super(Gaussian, self).__init__(
+            link=link,
+            variance=Gaussian.variance,
+            check_link=check_link
+            )
 
     def _resid_dev(self, endog, mu):
         r"""
@@ -545,7 +592,7 @@ class Gaussian(Family):
             Deviance residuals as defined below.
 
         Notes
-        --------
+        -----
         .. math::
 
            resid\_dev_i = (endog_i - \mu_i) ** 2
@@ -637,6 +684,28 @@ class Gaussian(Family):
         resid *= np.sqrt(var_weights)
         return resid
 
+    def get_distribution(self, mu, scale, var_weights=1.):
+        r"""
+        Frozen Gaussian distribution instance for given parameters
+
+        Parameters
+        ----------
+        mu : ndarray
+            Usually but not always the fitted mean response variable.
+        scale : float
+            The scale parameter is required argument for get_distribution.
+        var_weights : array_like
+            1d array of variance (analytic) weights. The default is 1.
+
+        Returns
+        -------
+        distribution instance
+
+        """
+
+        scale_n = scale / var_weights
+        return stats.norm(loc=mu, scale=np.sqrt(scale_n))
+
 
 class Gamma(Family):
     """
@@ -648,6 +717,10 @@ class Gamma(Family):
         The default link for the Gamma family is the inverse link.
         Available links are log, identity, and inverse.
         See statsmodels.genmod.families.links for more information.
+    check_link : bool
+        If True (default), then and exception is raised if the link is invalid
+        for the family.
+        If False, then the link is not checked.
 
     Attributes
     ----------
@@ -662,14 +735,18 @@ class Gamma(Family):
     statsmodels.genmod.families.family.Family : Parent class for all links.
     :ref:`links` : Further details on links.
     """
-    links = [L.log, L.identity, L.inverse_power]
+    links = [L.Log, L.Identity, L.InversePower]
     variance = V.mu_squared
     safe_links = [L.Log, ]
 
-    def __init__(self, link=None):
+    def __init__(self, link=None, check_link=True):
         if link is None:
-            link = L.inverse_power()
-        super(Gamma, self).__init__(link=link, variance=Gamma.variance)
+            link = L.InversePower()
+        super(Gamma, self).__init__(
+            link=link,
+            variance=Gamma.variance,
+            check_link=check_link
+            )
 
     def _resid_dev(self, endog, mu):
         r"""
@@ -771,6 +848,30 @@ class Gamma(Family):
         resid *= np.sqrt(var_weights)
         return resid
 
+    def get_distribution(self, mu, scale, var_weights=1.):
+        r"""
+        Frozen Gamma distribution instance for given parameters
+
+        Parameters
+        ----------
+        mu : ndarray
+            Usually but not always the fitted mean response variable.
+        scale : float
+            The scale parameter is required argument for get_distribution.
+        var_weights : array_like
+            1d array of variance (analytic) weights. The default is 1.
+
+        Returns
+        -------
+        distribution instance
+
+        """
+        # combine var_weights with scale
+        scale_ = scale / var_weights
+        shape = 1 / scale_
+        scale_g = mu * scale_
+        return stats.gamma(shape, scale=scale_g)
+
 
 class Binomial(Family):
     """
@@ -780,8 +881,12 @@ class Binomial(Family):
     ----------
     link : a link instance, optional
         The default link for the Binomial family is the logit link.
-        Available links are logit, probit, cauchy, log, and cloglog.
+        Available links are logit, probit, cauchy, log, loglog, and cloglog.
         See statsmodels.genmod.families.links for more information.
+    check_link : bool
+        If True (default), then and exception is raised if the link is invalid
+        for the family.
+        If False, then the link is not checked.
 
     Attributes
     ----------
@@ -809,22 +914,26 @@ class Binomial(Family):
     number of trials for each row.
     """
 
-    links = [L.logit, L.probit, L.cauchy, L.log, L.cloglog, L.identity]
+    links = [L.Logit, L.Probit, L.Cauchy, L.Log, L.LogC, L.CLogLog, L.LogLog,
+             L.Identity]
     variance = V.binary  # this is not used below in an effort to include n
 
     # Other safe links, e.g. cloglog and probit are subclasses
     safe_links = [L.Logit, L.CDFLink]
 
-    def __init__(self, link=None):  # , n=1.):
+    def __init__(self, link=None, check_link=True):  # , n=1.):
         if link is None:
-            link = L.logit()
+            link = L.Logit()
         # TODO: it *should* work for a constant n>1 actually, if freq_weights
         # is equal to n
         self.n = 1
         # overwritten by initialize if needed but always used to initialize
         # variance since endog is assumed/forced to be (0,1)
-        super(Binomial, self).__init__(link=link,
-                                       variance=V.Binomial(n=self.n))
+        super(Binomial, self).__init__(
+            link=link,
+            variance=V.Binomial(n=self.n),
+            check_link=check_link
+            )
 
     def starting_mu(self, y):
         r"""
@@ -890,8 +999,8 @@ class Binomial(Family):
            resid\_dev_i = 2 * n * (endog_i * \ln(endog_i /\mu_i) +
            (1 - endog_i) * \ln((1 - endog_i) / (1 - \mu_i)))
         """
-        endog_mu = self._clean(endog / mu)
-        n_endog_mu = self._clean((1. - endog) / (1. - mu))
+        endog_mu = self._clean(endog / (mu + 1e-20))
+        n_endog_mu = self._clean((1. - endog) / (1. - mu + 1e-20))
         resid_dev = endog * np.log(endog_mu) + (1 - endog) * np.log(n_endog_mu)
         return 2 * self.n * resid_dev
 
@@ -942,9 +1051,10 @@ class Binomial(Family):
         y = endog * n  # Number of successes
 
         # note that mu is still in (0,1), i.e. not converted back
-        return (special.gammaln(n + 1) - special.gammaln(y + 1) -
-                special.gammaln(n - y + 1) + y * np.log(mu / (1 - mu)) +
-                n * np.log(1 - mu)) * var_weights
+        return (
+            special.gammaln(n + 1) - special.gammaln(y + 1) -
+            special.gammaln(n - y + 1) + y * np.log(mu / (1 - mu + 1e-20)) +
+            n * np.log(1 - mu + 1e-20)) * var_weights
 
     def resid_anscombe(self, endog, mu, var_weights=1., scale=1.):
         r'''
@@ -1009,6 +1119,31 @@ class Binomial(Family):
         resid *= np.sqrt(var_weights)
         return resid
 
+    def get_distribution(self, mu, scale=1., var_weights=1., n_trials=1):
+        r"""
+        Frozen Binomial distribution instance for given parameters
+
+        Parameters
+        ----------
+        mu : ndarray
+            Usually but not always the fitted mean response variable.
+        scale : float
+            The scale parameter is ignored.
+        var_weights : array_like
+            1d array of variance (analytic) weights. The default is 1.
+            var_weights are ignored for Poisson.
+        n_trials : int
+            Number of trials for the binomial distribution. The default is 1
+            which corresponds to a Bernoulli random variable.
+
+        Returns
+        -------
+        distribution instance
+
+        """
+
+        return stats.binom(n=n_trials, p=mu)
+
 
 class InverseGaussian(Family):
     """
@@ -1019,8 +1154,12 @@ class InverseGaussian(Family):
     link : a link instance, optional
         The default link for the inverse Gaussian family is the
         inverse squared link.
-        Available links are inverse_squared, inverse, log, and identity.
+        Available links are InverseSquared, Inverse, Log, and Identity.
         See statsmodels.genmod.families.links for more information.
+    check_link : bool
+        If True (default), then and exception is raised if the link is invalid
+        for the family.
+        If False, then the link is not checked.
 
     Attributes
     ----------
@@ -1041,15 +1180,18 @@ class InverseGaussian(Family):
     literature as the Wald distribution.
     """
 
-    links = [L.inverse_squared, L.inverse_power, L.identity, L.log]
+    links = [L.InverseSquared, L.InversePower, L.Identity, L.Log]
     variance = V.mu_cubed
-    safe_links = [L.inverse_squared, L.Log, ]
+    safe_links = [L.InverseSquared, L.Log, ]
 
-    def __init__(self, link=None):
+    def __init__(self, link=None, check_link=True):
         if link is None:
-            link = L.inverse_squared()
+            link = L.InverseSquared()
         super(InverseGaussian, self).__init__(
-            link=link, variance=InverseGaussian.variance)
+            link=link,
+            variance=InverseGaussian.variance,
+            check_link=check_link
+            )
 
     def _resid_dev(self, endog, mu):
         r"""
@@ -1143,10 +1285,33 @@ class InverseGaussian(Family):
         resid *= np.sqrt(var_weights)
         return resid
 
+    def get_distribution(self, mu, scale, var_weights=1.):
+        r"""
+        Frozen Inverse Gaussian distribution instance for given parameters
+
+        Parameters
+        ----------
+        mu : ndarray
+            Usually but not always the fitted mean response variable.
+        scale : float
+            The scale parameter is required argument for get_distribution.
+        var_weights : array_like
+            1d array of variance (analytic) weights. The default is 1.
+
+        Returns
+        -------
+        distribution instance
+
+        """
+        # combine var_weights with scale
+        scale_ = scale / var_weights
+        mu_ig = mu * scale_
+        return stats.invgauss(mu_ig, scale=1 / scale_)
+
 
 class NegativeBinomial(Family):
     r"""
-    Negative Binomial exponential family.
+    Negative Binomial exponential family (corresponds to NB2).
 
     Parameters
     ----------
@@ -1158,6 +1323,10 @@ class NegativeBinomial(Family):
         The ancillary parameter for the negative binomial distribution.
         For now ``alpha`` is assumed to be nonstochastic.  The default value
         is 1.  Permissible values are usually assumed to be between .01 and 2.
+    check_link : bool
+        If True (default), then and exception is raised if the link is invalid
+        for the family.
+        If False, then the link is not checked.
 
     Attributes
     ----------
@@ -1186,18 +1355,25 @@ class NegativeBinomial(Family):
 
     with :math:`E[Y]=\mu\,` and :math:`Var[Y]=\mu+\alpha\mu^2`.
     """
-    links = [L.log, L.cloglog, L.identity, L.nbinom, L.Power]
+    links = [L.Log, L.CLogLog, L.Identity, L.NegativeBinomial, L.Power]
     # TODO: add the ability to use the power links with an if test
     # similar to below
     variance = V.nbinom
     safe_links = [L.Log, ]
 
-    def __init__(self, link=None, alpha=1.):
+    def __init__(self, link=None, alpha=1., check_link=True):
         self.alpha = 1. * alpha  # make it at least float
+        if alpha is self.__init__.__defaults__[1]:  # `is` is intentional
+            warnings.warn("Negative binomial dispersion parameter alpha not "
+                          f"set. Using default value alpha={alpha}.",
+                          ValueWarning)
         if link is None:
-            link = L.log()
+            link = L.Log()
         super(NegativeBinomial, self).__init__(
-            link=link, variance=V.NegativeBinomial(alpha=self.alpha))
+            link=link,
+            variance=V.NegativeBinomial(alpha=self.alpha),
+            check_link=check_link
+            )
 
     def _resid_dev(self, endog, mu):
         r"""
@@ -1329,6 +1505,29 @@ class NegativeBinomial(Family):
         resid *= np.sqrt(var_weights)
         return resid
 
+    def get_distribution(self, mu, scale=1., var_weights=1.):
+        r"""
+        Frozen NegativeBinomial distribution instance for given parameters
+
+        Parameters
+        ----------
+        mu : ndarray
+            Usually but not always the fitted mean response variable.
+        scale : float
+            The scale parameter is ignored.
+        var_weights : array_like
+            1d array of variance (analytic) weights. The default is 1.
+            var_weights are ignored for NegativeBinomial.
+
+        Returns
+        -------
+        distribution instance
+
+        """
+        size = 1. / self.alpha
+        prob = size / (size + mu)
+        return stats.nbinom(size, prob)
+
 
 class Tweedie(Family):
     """
@@ -1338,14 +1537,19 @@ class Tweedie(Family):
     ----------
     link : a link instance, optional
         The default link for the Tweedie family is the log link.
-        Available links are log and Power.
+        Available links are log, Power and any aliases of power.
         See statsmodels.genmod.families.links for more information.
     var_power : float, optional
         The variance power. The default is 1.
     eql : bool
         If True, the Extended Quasi-Likelihood is used, else the
-        likelihood is used (however the latter is not implemented).
-        If eql is True, var_power must be between 1 and 2.
+        likelihood is used.
+        In both cases, for likelihood computations the var_power
+        must be between 1 and 2.
+    check_link : bool
+        If True (default), then and exception is raised if the link is invalid
+        for the family.
+        If False, then the link is not checked.
 
     Attributes
     ----------
@@ -1355,7 +1559,7 @@ class Tweedie(Family):
         ``variance`` is an instance of
         statsmodels.genmod.families.varfuncs.Power
     Tweedie.var_power : float
-        The power of the variance function.
+        The power parameter of the variance function.
 
     See Also
     --------
@@ -1369,20 +1573,23 @@ class Tweedie(Family):
     estimated using the ``estimate_tweedie_power`` function that is part of the
     statsmodels.genmod.generalized_linear_model.GLM class.
     """
-    links = [L.log, L.Power]
+    links = [L.Log, L.Power]
     variance = V.Power(power=1.5)
-    safe_links = [L.log, L.Power]
+    safe_links = [L.Log, L.Power]
 
-    def __init__(self, link=None, var_power=1., eql=False):
+    def __init__(self, link=None, var_power=1., eql=False, check_link=True):
         self.var_power = var_power
         self.eql = eql
         if eql and (var_power < 1 or var_power > 2):
             raise ValueError("Tweedie: if EQL=True then var_power must fall "
                              "between 1 and 2")
         if link is None:
-            link = L.log()
+            link = L.Log()
         super(Tweedie, self).__init__(
-            link=link, variance=V.Power(power=var_power * 1.))
+            link=link,
+            variance=V.Power(power=var_power * 1.),
+            check_link=check_link
+            )
 
     def _resid_dev(self, endog, mu):
         r"""
@@ -1479,33 +1686,81 @@ class Tweedie(Family):
 
         References
         ----------
+        R Kaas (2005).  Compound Poisson Distributions and GLM's -- Tweedie's
+        Distribution.
+        https://core.ac.uk/download/pdf/6347266.pdf#page=11
+
         JA Nelder, D Pregibon (1987).  An extended quasi-likelihood function.
         Biometrika 74:2, pp 221-232.  https://www.jstor.org/stable/2336136
         """
-        if not self.eql:
-            # We have not yet implemented the actual likelihood
-            return np.nan
-
-        # Equations 9-10 or Nelder and Pregibon
         p = self.var_power
-        llf = np.log(2 * np.pi * scale) + p * np.log(mu) - np.log(var_weights)
-        llf /= -2
-
+        endog = np.atleast_1d(endog)
         if p == 1:
-            u = endog * np.log(endog / mu) - (endog - mu)
-            u *= var_weights / scale
+            return Poisson().loglike_obs(
+                endog=endog,
+                mu=mu,
+                var_weights=var_weights,
+                scale=scale
+            )
         elif p == 2:
-            yr = endog / mu
-            u = yr - np.log(yr) - 1
-            u *= var_weights / scale
+            return Gamma().loglike_obs(
+                endog=endog,
+                mu=mu,
+                var_weights=var_weights,
+                scale=scale
+            )
+
+        if not self.eql:
+            if p < 1 or p > 2:
+                # We have not yet implemented the actual likelihood
+                return np.nan
+
+            # scipy compat bessel_wright added in 1.7
+            if SP_LT_17:
+                # old return was nan
+                return np.nan
+
+            # See: Dunn, Smyth (2004) "Series evaluation of Tweedie
+            # exponential dispersion model densities"
+            # pdf(y, mu, p, phi) = f(y, theta, phi)
+            # = c(y, phi) * exp(1/phi (y theta - kappa(theta)))
+            # kappa = cumulant function
+            # theta = function of expectation mu and power p
+            # alpha = (2-p)/(1-p)
+            # phi = scale
+            # for 1<p<2:
+            # c(y, phi) = 1/y * wright_bessel(a, b, x)
+            # a = -alpha
+            # b = 0
+            # x = (p-1)**alpha/(2-p) / y**alpha / phi**(1-alpha)
+            scale = scale / var_weights
+            theta = mu ** (1 - p) / (1 - p)
+            kappa = mu ** (2 - p) / (2 - p)
+            alpha = (2 - p) / (1 - p)
+
+            ll_obs = (endog * theta - kappa) / scale
+            idx = endog > 0
+            if np.any(idx):
+                if not np.isscalar(endog):
+                    endog = endog[idx]
+                if not np.isscalar(scale):
+                    scale = scale[idx]
+                x = ((p - 1) * scale / endog) ** alpha
+                x /= (2 - p) * scale
+                wb = special.wright_bessel(-alpha, 0, x)
+                ll_obs[idx] += np.log(1/endog * wb)
+            return ll_obs
         else:
+            # Equations 4 of Kaas
+            llf = np.log(2 * np.pi * scale) + p * np.log(endog)
+            llf -= np.log(var_weights)
+            llf /= -2
             u = (endog ** (2 - p)
                  - (2 - p) * endog * mu ** (1 - p)
                  + (1 - p) * mu ** (2 - p))
             u *= var_weights / (scale * (1 - p) * (2 - p))
-        llf -= u
 
-        return llf
+        return llf - u
 
     def resid_anscombe(self, endog, mu, var_weights=1., scale=1.):
         r"""
