@@ -53,17 +53,18 @@ def gendat():
 
     df["y"] = endog
 
-    df.x1[0:60] = np.nan
-    df.x2[0:40] = np.nan
-    df.x3[10:30:2] = np.nan
-    df.x4[20:50:3] = np.nan
-    df.x5[40:45] = np.nan
-    df.y[30:100:2] = np.nan
+    # loc is inclusive of right end, so needed to lower index by 1
+    df.loc[0:59, "x1"] = np.nan
+    df.loc[0:39, "x2"] = np.nan
+    df.loc[10:29:2, "x3"] = np.nan
+    df.loc[20:49:3, "x4"] = np.nan
+    df.loc[40:44, "x5"] = np.nan
+    df.loc[30:99:2, "y"] = np.nan
 
     return df
 
 
-class TestMICEData(object):
+class TestMICEData:
 
     def test_default(self):
         # Test with all defaults.
@@ -121,7 +122,9 @@ class TestMICEData(object):
             with warnings.catch_warnings(record=True) as ws:
                 warnings.simplefilter('always')
                 miceData.update_all()
-
+                # Only include pandas warnings. There are many from patsy
+                # and sometimes warnings from other packages here
+                ws = [w for w in ws if "\\pandas\\" in w.filename]
                 assert len(ws) == 0
 
     def test_next_sample(self):
@@ -298,7 +301,7 @@ class TestMICEData(object):
             close_figures()
 
 
-class TestMICE(object):
+class TestMICE:
 
     def test_MICE(self):
 
@@ -349,7 +352,7 @@ class TestMICE(object):
             assert isinstance(x.family, sm.families.Binomial)
 
     @pytest.mark.slow
-    def test_combine(self):
+    def t_est_combine(self):
 
         np.random.seed(3897)
         x1 = np.random.normal(size=300)
