@@ -30,12 +30,33 @@ refactoring
 
 
 """
-from __future__ import print_function
-from statsmodels.compat.python import iteritems
+import warnings
+
 import numpy as np
-from scipy import stats, optimize
+from scipy import stats, optimize, special
 from statsmodels.tools.rootfinding import brentq_expanding
 
+
+def nct_cdf(x, df, nc):
+    return special.nctdtr(df, nc, x)
+
+
+def nct_sf(x, df, nc):
+    return 1 - special.nctdtr(df, nc, x)
+
+
+def ncf_cdf(x, dfn, dfd, nc):
+    return special.ncfdtr(dfn, dfd, nc, x)
+
+
+def ncf_sf(x, dfn, dfd, nc):
+    return 1 - special.ncfdtr(dfn, dfd, nc, x)
+
+
+def ncf_ppf(q, dfn, dfd, nc):
+    return special.ncfdtri(dfn, dfd, nc, q)
+
+
 def ttest_power(effect_size, nobs, alpha, df=None, alternative='two-sided'):
     '''Calculate power of a ttest
     '''
@@ -44,7 +65,7 @@ def ttest_power(effect_size, nobs, alpha, df=None, alternative='two-sided'):
         df = nobs - 1
 
     if alternative in ['two-sided', '2s']:
-        alpha_ = alpha / 2.  #no inplace changes, doesn't work
+        alpha_ = alpha / 2.  #no inplace changes, does not work
     elif alternative in ['smaller', 'larger']:
         alpha_ = alpha
     else:
@@ -60,24 +81,46 @@ def ttest_power(effect_size, nobs, alpha, df=None, alternative='two-sided'):
             # avoid endless loop, https://github.com/scipy/scipy/issues/2667
             pow_ = np.nan
         else:
-            pow_ = stats.nct._sf(crit_upp, df, d*np.sqrt(nobs))
+            # pow_ = stats.nct._sf(crit_upp, df, d*np.sqrt(nobs))
+            # use scipy.special
+            pow_ = nct_sf(crit_upp, df, d*np.sqrt(nobs))
     if alternative in ['two-sided', '2s', 'smaller']:
         crit_low = stats.t.ppf(alpha_, df)
         #print crit_low, df, d*np.sqrt(nobs)
         if np.any(np.isnan(crit_low)):
             pow_ = np.nan
         else:
-            pow_ += stats.nct._cdf(crit_low, df, d*np.sqrt(nobs))
+            # pow_ += stats.nct._cdf(crit_low, df, d*np.sqrt(nobs))
+            pow_ += nct_cdf(crit_low, df, d*np.sqrt(nobs))
     return pow_
 
-def normal_power(effect_size, nobs, alpha, alternative='two-sided', sigma=1.):
-    '''Calculate power of a normal distributed test statistic
 
-    '''
+def normal_power(effect_size, nobs, alpha, alternative='two-sided', sigma=1.):
+    """Calculate power of a normal distributed test statistic
+
+    This is an generalization of `normal_power` when variance under Null and
+    Alternative differ.
+
+    Parameters
+    ----------
+    effect size : float
+        difference in the estimated means or statistics under the alternative
+        normalized by the standard deviation (without division by sqrt(nobs).
+    nobs : float or int
+        number of observations
+    alpha : float in interval (0,1)
+        significance level, e.g. 0.05, is the probability of a type I
+        error, that is wrong rejections if the Null Hypothesis is true.
+    alternative : string, 'two-sided' (default), 'larger', 'smaller'
+        extra argument to choose whether the power is calculated for a
+        two-sided (default) or one sided test. The one-sided test can be
+        either 'larger', 'smaller'.
+    """
+
     d = effect_size
 
     if alternative in ['two-sided', '2s']:
-        alpha_ = alpha / 2.  #no inplace changes, doesn't work
+        alpha_ = alpha / 2.  #no inplace changes, does not work
     elif alternative in ['smaller', 'larger']:
         alpha_ = alpha
     else:
@@ -93,6 +136,115 @@ def normal_power(effect_size, nobs, alpha, alternative='two-sided', sigma=1.):
         pow_ += stats.norm.cdf(crit - d*np.sqrt(nobs)/sigma)
     return pow_
 
+
+def normal_power_het(diff, nobs, alpha, std_null=1., std_alternative=None,
+                 alternative='two-sided'):
+    """Calculate power of a normal distributed test statistic
+
+    This is an generalization of `normal_power` when variance under Null and
+    Alternative differ.
+
+    Parameters
+    ----------
+    diff : float
+        difference in the estimated means or statistics under the alternative.
+    nobs : float or int
+        number of observations
+    alpha : float in interval (0,1)
+        significance level, e.g. 0.05, is the probability of a type I
+        error, that is wrong rejections if the Null Hypothesis is true.
+    std_null : float
+        standard deviation under the Null hypothesis without division by
+        sqrt(nobs)
+    std_alternative : float
+        standard deviation under the Alternative hypothesis without division
+        by sqrt(nobs)
+    alternative : string, 'two-sided' (default), 'larger', 'smaller'
+        extra argument to choose whether the power is calculated for a
+        two-sided (default) or one sided test. The one-sided test can be
+        either 'larger', 'smaller'.
+
+    Returns
+    -------
+    power : float
+    """
+
+    d = diff
+    if std_alternative is None:
+        std_alternative = std_null
+
+    if alternative in ['two-sided', '2s']:
+        alpha_ = alpha / 2.  #no inplace changes, does not work
+    elif alternative in ['smaller', 'larger']:
+        alpha_ = alpha
+    else:
+        raise ValueError("alternative has to be 'two-sided', 'larger' " +
+                         "or 'smaller'")
+
+    std_ratio = std_null / std_alternative
+    pow_ = 0
+    if alternative in ['two-sided', '2s', 'larger']:
+        crit = stats.norm.isf(alpha_)
+        pow_ = stats.norm.sf(crit * std_ratio -
+                             d*np.sqrt(nobs) / std_alternative)
+    if alternative in ['two-sided', '2s', 'smaller']:
+        crit = stats.norm.ppf(alpha_)
+        pow_ += stats.norm.cdf(crit * std_ratio -
+                               d*np.sqrt(nobs) / std_alternative)
+    return pow_
+
+
+def normal_sample_size_one_tail(diff, power, alpha, std_null=1.,
+                                std_alternative=None):
+    """explicit sample size computation if only one tail is relevant
+
+    The sample size is based on the power in one tail assuming that the
+    alternative is in the tail where the test has power that increases
+    with sample size.
+    Use alpha/2 to compute the one tail approximation to the two-sided
+    test, i.e. consider only one tail of two-sided test.
+
+    Parameters
+    ----------
+    diff : float
+        difference in the estimated means or statistics under the alternative.
+    power : float in interval (0,1)
+        power of the test, e.g. 0.8, is one minus the probability of a type II
+        error. Power is the probability that the test correctly rejects the
+        Null Hypothesis if the Alternative Hypothesis is true.
+    alpha : float in interval (0,1)
+        significance level, e.g. 0.05, is the probability of a type I
+        error, that is wrong rejections if the Null Hypothesis is true.
+        Note: alpha is used for one tail. Use alpha/2 for two-sided
+        alternative.
+    std_null : float
+        standard deviation under the Null hypothesis without division by
+        sqrt(nobs)
+    std_alternative : float
+        standard deviation under the Alternative hypothesis without division
+        by sqrt(nobs). Defaults to None. If None, ``std_alternative`` is set
+        to the value of ``std_null``.
+
+    Returns
+    -------
+    nobs : float
+        Sample size to achieve (at least) the desired power.
+        If the minimum power is satisfied for all positive sample sizes, then
+        ``nobs`` will be zero. This will be the case when power <= alpha if
+        std_alternative is equal to std_null.
+
+    """
+
+    if std_alternative is None:
+        std_alternative = std_null
+
+    crit_power = stats.norm.isf(power)
+    crit = stats.norm.isf(alpha)
+    n1 = (np.maximum(crit * std_null - crit_power * std_alternative, 0)
+          / diff)**2
+    return n1
+
+
 def ftest_anova_power(effect_size, nobs, alpha, k_groups=2, df=None):
     '''power for ftest for one way anova with k equal sized groups
 
@@ -100,24 +252,26 @@ def ftest_anova_power(effect_size, nobs, alpha, k_groups=2, df=None):
 
     should be general nobs observations, k_groups restrictions ???
     '''
-    df_num = nobs - k_groups
-    df_denom = k_groups - 1
-    crit = stats.f.isf(alpha, df_denom, df_num)
-    pow_ = stats.ncf.sf(crit, df_denom, df_num, effect_size**2 * nobs)
-    return pow_#, crit
+    df_num = k_groups - 1
+    df_denom = nobs - k_groups
+    crit = stats.f.isf(alpha, df_num, df_denom)
+    pow_ = ncf_sf(crit, df_num, df_denom, effect_size**2 * nobs)
+    return pow_
 
-def ftest_power(effect_size, df_num, df_denom, alpha, ncc=1):
+
+def ftest_power(effect_size, df2, df1, alpha, ncc=1):
     '''Calculate the power of a F-test.
 
     Parameters
     ----------
     effect_size : float
-        standardized effect size, mean divided by the standard deviation.
-        effect size has to be positive.
-    df_num : int or float
-        numerator degrees of freedom.
-    df_denom : int or float
-        denominator degrees of freedom.
+        The effect size is here Cohen's ``f``, the square root of ``f2``.
+    df2 : int or float
+        Denominator degrees of freedom.
+        This corresponds to the df_resid in Wald tests.
+    df1 : int or float
+        Numerator degrees of freedom.
+        This corresponds to the number of constraints in Wald tests.
     alpha : float in interval (0,1)
         significance level, e.g. 0.05, is the probability of a type I
         error, that is wrong rejections if the Null Hypothesis is true.
@@ -134,25 +288,95 @@ def ftest_power(effect_size, df_num, df_denom, alpha, ncc=1):
 
     Notes
     -----
+    changed in 0.14: use df2, df1 instead of df_num, df_denom as arg names.
+    The latter had reversed meaning.
 
-    sample size is given implicitly by df_num
+    The sample size is given implicitly by ``df2`` with fixed number of
+    constraints given by numerator degrees of freedom ``df1``:
 
-    set ncc=0 to match t-test, or f-test in LikelihoodModelResults.
+        nobs = df2 + df1 + ncc
+
+    Set ncc=0 to match t-test, or f-test in LikelihoodModelResults.
     ncc=1 matches the non-centrality parameter in R::pwr::pwr.f2.test
 
     ftest_power with ncc=0 should also be correct for f_test in regression
-    models, with df_num and d_denom as defined there. (not verified yet)
+    models, with df_num (df1) as number of constraints and d_denom (df2) as
+    df_resid.
     '''
+    df_num, df_denom = df1, df2
     nc = effect_size**2 * (df_denom + df_num + ncc)
-    crit = stats.f.isf(alpha, df_denom, df_num)
-    pow_ = stats.ncf.sf(crit, df_denom, df_num, nc)
+    crit = stats.f.isf(alpha, df_num, df_denom)
+    # pow_ = stats.ncf.sf(crit, df_num, df_denom, nc)
+    # use scipy.special for ncf
+    pow_ = ncf_sf(crit, df_num, df_denom, nc)
     return pow_ #, crit, nc
 
 
+def ftest_power_f2(effect_size, df_num, df_denom, alpha, ncc=1):
+    '''Calculate the power of a F-test.
+
+    Based on Cohen's `f^2` effect size.
+
+    This assumes
+
+        df_num : numerator degrees of freedom, (number of constraints)
+        df_denom : denominator degrees of freedom (df_resid in regression)
+        nobs = df_denom + df_num + ncc
+        nc = effect_size * nobs  (noncentrality index)
+
+    Power is computed one-sided in the upper tail.
+
+    Parameters
+    ----------
+    effect_size : float
+        Cohen's f2 effect size or noncentrality divided by nobs.
+    df_num : int or float
+        Numerator degrees of freedom.
+        This corresponds to the number of constraints in Wald tests.
+    df_denom : int or float
+        Denominator degrees of freedom.
+        This corresponds to the df_resid in Wald tests.
+    alpha : float in interval (0,1)
+        significance level, e.g. 0.05, is the probability of a type I
+        error, that is wrong rejections if the Null Hypothesis is true.
+    ncc : int
+        degrees of freedom correction for non-centrality parameter.
+        see Notes
+
+    Returns
+    -------
+    power : float
+        Power of the test, e.g. 0.8, is one minus the probability of a
+        type II error. Power is the probability that the test correctly
+        rejects the Null Hypothesis if the Alternative Hypothesis is true.
+
+    Notes
+
+    The sample size is given implicitly by ``df_denom`` with fixed number of
+    constraints given by numerator degrees of freedom ``df_num``:
+
+        nobs = df_denom + df_num + ncc
+
+    Set ncc=0 to match t-test, or f-test in LikelihoodModelResults.
+    ncc=1 matches the non-centrality parameter in R::pwr::pwr.f2.test
+
+    ftest_power with ncc=0 should also be correct for f_test in regression
+    models, with df_num (df1) as number of constraints and d_denom (df2) as
+    df_resid.
+    '''
+
+    nc = effect_size * (df_denom + df_num + ncc)
+    crit = stats.f.isf(alpha, df_num, df_denom)
+    # pow_ = stats.ncf.sf(crit, df_num, df_denom, nc)
+    # use scipy.special for ncf
+    pow_ = ncf_sf(crit, df_num, df_denom, nc)
+    return pow_
+
+
 #class based implementation
 #--------------------------
 
-class Power(object):
+class Power:
     '''Statistical Power calculations, Base Class
 
     so far this could all be class methods
@@ -209,8 +433,8 @@ class Power(object):
         #TODO: maybe use explicit kwds,
         #    nicer but requires inspect? and not generic across tests
         #    I'm duplicating this in the subclass to get informative docstring
-        key = [k for k,v in iteritems(kwds) if v is None]
-        #print kwds, key;
+        key = [k for k,v in kwds.items() if v is None]
+        #print kwds, key
         if len(key) != 1:
             raise ValueError('need exactly one keyword that is None')
         key = key[0]
@@ -219,7 +443,20 @@ class Power(object):
             del kwds['power']
             return self.power(**kwds)
 
+        if kwds['effect_size'] == 0:
+            import warnings
+            from statsmodels.tools.sm_exceptions import HypothesisTestWarning
+            warnings.warn('Warning: Effect size of 0 detected', HypothesisTestWarning)
+            if key == 'power':
+                return kwds['alpha']
+            if key == 'alpha':
+                return kwds['power']
+            else:
+                raise ValueError('Cannot detect an effect-size of 0. Try changing your effect-size.')
+
+
         self._counter = 0
+
         def func(x):
             kwds[key] = x
             fval = self._power_identity(**kwds)
@@ -237,7 +474,9 @@ class Power(object):
             start_value = self.start_ttp[key]
         except KeyError:
             start_value = 0.9
-            print('Warning: using default start_value for {0}'.format(key))
+            import warnings
+            from statsmodels.tools.sm_exceptions import ValueWarning
+            warnings.warn('Warning: using default start_value for {0}'.format(key), ValueWarning)
 
         fit_kwds = self.start_bqexp[key]
         fit_res = []
@@ -255,12 +494,18 @@ class Power(object):
             success = 1
         else:
             # try backup
-            #TODO: check more cases to make this robust
-            val, infodict, ier, msg = optimize.fsolve(func, start_value,
-                                                      full_output=True) #scalar
-            #val = optimize.newton(func, start_value) #scalar
-            fval = infodict['fvec']
-            fit_res.append(infodict)
+            # TODO: check more cases to make this robust
+            if not np.isnan(start_value):
+                val, infodict, ier, msg = optimize.fsolve(func, start_value,
+                                                          full_output=True) #scalar
+                #val = optimize.newton(func, start_value) #scalar
+                fval = infodict['fvec']
+                fit_res.append(infodict)
+            else:
+                ier = -1
+                fval = 1
+                fit_res.append([None])
+
             if ier == 1 and np.abs(fval) < 1e-4 :
                 success = 1
             else:
@@ -286,38 +531,41 @@ class Power(object):
 
     def plot_power(self, dep_var='nobs', nobs=None, effect_size=None,
                    alpha=0.05, ax=None, title=None, plt_kwds=None, **kwds):
-        '''plot power with number of observations or effect size on x-axis
+        """
+        Plot power with number of observations or effect size on x-axis
 
         Parameters
         ----------
-        dep_var : string in ['nobs', 'effect_size', 'alpha']
+        dep_var : {'nobs', 'effect_size', 'alpha'}
             This specifies which variable is used for the horizontal axis.
             If dep_var='nobs' (default), then one curve is created for each
             value of ``effect_size``. If dep_var='effect_size' or alpha, then
             one curve is created for each value of ``nobs``.
-        nobs : scalar or array_like
+        nobs : {scalar, array_like}
             specifies the values of the number of observations in the plot
-        effect_size : scalar or array_like
+        effect_size : {scalar, array_like}
             specifies the values of the effect_size in the plot
-        alpha : float or array_like
+        alpha : {float, array_like}
             The significance level (type I error) used in the power
             calculation. Can only be more than a scalar, if ``dep_var='alpha'``
         ax : None or axis instance
             If ax is None, than a matplotlib figure is created. If ax is a
             matplotlib axis instance, then it is reused, and the plot elements
             are created with it.
-        title : string
+        title : str
             title for the axis. Use an empty string, ``''``, to avoid a title.
-        plt_kwds : None or dict
+        plt_kwds : {None, dict}
             not used yet
-        kwds : optional keywords for power function
+        kwds : dict
             These remaining keyword arguments are used as arguments to the
             power function. Many power function support ``alternative`` as a
             keyword argument, two-sample test support ``ratio``.
 
         Returns
         -------
-        fig : matplotlib figure instance
+        Figure
+            If `ax` is None, the created figure.  Otherwise the figure to which
+            `ax` is connected.
 
         Notes
         -----
@@ -328,7 +576,7 @@ class Power(object):
         TODO: fix this for FTestPower and GofChisquarePower
 
         TODO: maybe add line variable, if we want more than nobs and effectsize
-        '''
+        """
         #if pwr_kwds is None:
         #    pwr_kwds = {}
         from statsmodels.graphics import utils
@@ -395,7 +643,7 @@ class TTestPower(Power):
         df : int or float
             degrees of freedom. By default this is None, and the df from the
             one sample or paired ttest is used, ``df = nobs1 - 1``
-        alternative : string, 'two-sided' (default), 'larger', 'smaller'
+        alternative : str, 'two-sided' (default), 'larger', 'smaller'
             extra argument to choose whether the power is calculated for a
             two-sided (default) or one sided test. The one-sided test can be
             either 'larger', 'smaller'.
@@ -431,8 +679,8 @@ class TTestPower(Power):
         Parameters
         ----------
         effect_size : float
-            standardized effect size, mean divided by the standard deviation.
-            effect size has to be positive.
+            Standardized effect size.The effect size is here Cohen's f, square
+            root of "f2".
         nobs : int or float
             sample size, number of observations.
         alpha : float in interval (0,1)
@@ -442,7 +690,7 @@ class TTestPower(Power):
             power of the test, e.g. 0.8, is one minus the probability of a
             type II error. Power is the probability that the test correctly
             rejects the Null Hypothesis if the Alternative Hypothesis is true.
-        alternative : string, 'two-sided' (default) or 'one-sided'
+        alternative : str, 'two-sided' (default) or 'one-sided'
             extra argument to choose whether the power is calculated for a
             two-sided (default) or one sided test.
             'one-sided' assumes we are in the relevant tail.
@@ -512,7 +760,7 @@ class TTestIndPower(Power):
         df : int or float
             degrees of freedom. By default this is None, and the df from the
             ttest with pooled variance is used, ``df = (nobs1 - 1 + nobs2 - 1)``
-        alternative : string, 'two-sided' (default), 'larger', 'smaller'
+        alternative : str, 'two-sided' (default), 'larger', 'smaller'
             extra argument to choose whether the power is calculated for a
             two-sided (default) or one sided test. The one-sided test can be
             either 'larger', 'smaller'.
@@ -566,7 +814,7 @@ class TTestIndPower(Power):
             sample 1. see description of nobs1
             The default for ratio is 1; to solve for ratio given the other
             arguments it has to be explicitly set to None.
-        alternative : string, 'two-sided' (default), 'larger', 'smaller'
+        alternative : str, 'two-sided' (default), 'larger', 'smaller'
             extra argument to choose whether the power is calculated for a
             two-sided (default) or one sided test. The one-sided test can be
             either 'larger', 'smaller'.
@@ -608,7 +856,7 @@ class NormalIndPower(Power):
 
     def power(self, effect_size, nobs1, alpha, ratio=1,
               alternative='two-sided'):
-        '''Calculate the power of a t-test for two independent sample
+        '''Calculate the power of a z-test for two independent sample
 
         Parameters
         ----------
@@ -627,9 +875,7 @@ class NormalIndPower(Power):
         ratio : float
             ratio of the number of observations in sample 2 relative to
             sample 1. see description of nobs1
-            The default for ratio is 1; to solve for ratio given the other
-            arguments it has to be explicitly set to None.
-        alternative : string, 'two-sided' (default), 'larger', 'smaller'
+        alternative : str, 'two-sided' (default), 'larger', 'smaller'
             extra argument to choose whether the power is calculated for a
             two-sided (default) or one sided test. The one-sided test can be
             either 'larger', 'smaller'.
@@ -689,7 +935,7 @@ class NormalIndPower(Power):
             sample 1. see description of nobs1
             The default for ratio is 1; to solve for ration given the other
             arguments it has to be explicitly set to None.
-        alternative : string, 'two-sided' (default), 'larger', 'smaller'
+        alternative : str, 'two-sided' (default), 'larger', 'smaller'
             extra argument to choose whether the power is calculated for a
             two-sided (default) or one sided test. The one-sided test can be
             either 'larger', 'smaller'.
@@ -720,22 +966,70 @@ class NormalIndPower(Power):
 
 
 class FTestPower(Power):
-    '''Statistical Power calculations for generic F-test
+    """Statistical Power calculations for generic F-test of a constraint
 
-    '''
+    This class is not recommended, use `FTestPowerF2` with corrected interface.
+
+    This is based on Cohen's f as effect size measure.
+
+    Warning: Methods in this class have the names df_num and df_denom reversed.
+
+    See Also
+    --------
+    FTestPowerF2 :
+        Class with Cohen's f-squared as effect size, corrected keyword names.
+
+    Examples
+    --------
+    Sample size and power for multiple regression base on R-squared
+
+    Compute effect size from R-squared
+
+    >>> r2 = 0.1
+    >>> f2 = r2 / (1 - r2)
+    >>> f = np.sqrt(f2)
+    >>> r2, f2, f
+    (0.1, 0.11111111111111112, 0.33333333333333337)
+
+    Find sample size by solving for denominator df, wrongly named ``df_num``
+
+    >>> df1 = 1  # number of constraints in hypothesis test
+    >>> df2 = FTestPower().solve_power(effect_size=f, alpha=0.1, power=0.9,
+                                       df_denom=df1)
+    >>> ncc = 1  # default
+    >>> nobs = df2 + df1 + ncc
+    >>> df2, nobs
+    (76.46459758305376, 78.46459758305376)
+
+    verify power at df2
+
+    >>> FTestPower().power(effect_size=f, alpha=0.1, df_denom=df1, df_num=df2)
+    0.8999999972109698
+
+    """
 
     def power(self, effect_size, df_num, df_denom, alpha, ncc=1):
         '''Calculate the power of a F-test.
 
+        The effect size is Cohen's ``f``, square root of ``f2``.
+
+        The sample size is given by ``nobs = df_denom + df_num + ncc``
+
+        Warning: The meaning of df_num and df_denom is reversed.
+
         Parameters
         ----------
         effect_size : float
-            standardized effect size, mean divided by the standard deviation.
-            effect size has to be positive.
+            Standardized effect size. The effect size is here Cohen's ``f``,
+            square root of ``f2``.
         df_num : int or float
-            numerator degrees of freedom.
+            Warning incorrect name
+            denominator degrees of freedom,
+            This corresponds to the number of constraints in Wald tests.
         df_denom : int or float
-            denominator degrees of freedom.
+            Warning incorrect name
+            numerator degrees of freedom.
+            This corresponds to the df_resid in Wald tests.
         alpha : float in interval (0,1)
             significance level, e.g. 0.05, is the probability of a type I
             error, that is wrong rejections if the Null Hypothesis is true.
@@ -768,7 +1062,7 @@ class FTestPower(Power):
 
     #method is only added to have explicit keywords and docstring
     def solve_power(self, effect_size=None, df_num=None, df_denom=None,
-                    nobs=None, alpha=None, power=None, ncc=1):
+                    alpha=None, power=None, ncc=1, **kwargs):
         '''solve for any one parameter of the power of a F-test
 
         for the one sample F-test the keywords are:
@@ -776,14 +1070,26 @@ class FTestPower(Power):
 
         Exactly one needs to be ``None``, all others need numeric values.
 
+        The effect size is Cohen's ``f``, square root of ``f2``.
+
+        The sample size is given by ``nobs = df_denom + df_num + ncc``.
+
+        Warning: The meaning of df_num and df_denom is reversed.
 
         Parameters
         ----------
         effect_size : float
-            standardized effect size, mean divided by the standard deviation.
-            effect size has to be positive.
-        nobs : int or float
-            sample size, number of observations.
+            Standardized effect size. The effect size is here Cohen's ``f``,
+            square root of ``f2``.
+        df_num : int or float
+            Warning incorrect name
+            denominator degrees of freedom,
+            This corresponds to the number of constraints in Wald tests.
+            Sample size is given by ``nobs = df_denom + df_num + ncc``
+        df_denom : int or float
+            Warning incorrect name
+            numerator degrees of freedom.
+            This corresponds to the df_resid in Wald tests.
         alpha : float in interval (0,1)
             significance level, e.g. 0.05, is the probability of a type I
             error, that is wrong rejections if the Null Hypothesis is true.
@@ -791,16 +1097,167 @@ class FTestPower(Power):
             power of the test, e.g. 0.8, is one minus the probability of a
             type II error. Power is the probability that the test correctly
             rejects the Null Hypothesis if the Alternative Hypothesis is true.
-        alternative : string, 'two-sided' (default) or 'one-sided'
-            extra argument to choose whether the power is calculated for a
-            two-sided (default) or one sided test.
-            'one-sided' assumes we are in the relevant tail.
+        ncc : int
+            degrees of freedom correction for non-centrality parameter.
+            see Notes
+        kwargs : empty
+            ``kwargs`` are not used and included for backwards compatibility.
+            If ``nobs`` is used as keyword, then a warning is issued. All
+            other keywords in ``kwargs`` raise a ValueError.
 
         Returns
         -------
         value : float
             The value of the parameter that was set to None in the call. The
-            value solves the power equation given the remainding parameters.
+            value solves the power equation given the remaining parameters.
+
+
+        Notes
+        -----
+        The method uses scipy.optimize for finding the value that satisfies
+        the power equation. It first uses ``brentq`` with a prior search for
+        bounds. If this fails to find a root, ``fsolve`` is used. If ``fsolve``
+        also fails, then, for ``alpha``, ``power`` and ``effect_size``,
+        ``brentq`` with fixed bounds is used. However, there can still be cases
+        where this fails.
+
+        '''
+        if kwargs:
+            if "nobs" in kwargs:
+                warnings.warn("nobs is not used")
+            else:
+                raise ValueError(f"incorrect keyword(s) {kwargs}")
+        return super(FTestPower, self).solve_power(effect_size=effect_size,
+                                                      df_num=df_num,
+                                                      df_denom=df_denom,
+                                                      alpha=alpha,
+                                                      power=power,
+                                                      ncc=ncc)
+
+
+
+class FTestPowerF2(Power):
+    """Statistical Power calculations for generic F-test of a constraint
+
+    This is based on Cohen's f^2 as effect size measure.
+
+    Examples
+    --------
+    Sample size and power for multiple regression base on R-squared
+
+    Compute effect size from R-squared
+
+    >>> r2 = 0.1
+    >>> f2 = r2 / (1 - r2)
+    >>> f = np.sqrt(f2)
+    >>> r2, f2, f
+    (0.1, 0.11111111111111112, 0.33333333333333337)
+
+    Find sample size by solving for denominator degrees of freedom.
+
+    >>> df1 = 1  # number of constraints in hypothesis test
+    >>> df2 = FTestPowerF2().solve_power(effect_size=f2, alpha=0.1, power=0.9,
+                                         df_num=df1)
+    >>> ncc = 1  # default
+    >>> nobs = df2 + df1 + ncc
+    >>> df2, nobs
+    (76.46459758305376, 78.46459758305376)
+
+    verify power at df2
+
+    >>> FTestPowerF2().power(effect_size=f, alpha=0.1, df_num=df1, df_denom=df2)
+    0.8999999972109698
+
+    """
+
+    def power(self, effect_size, df_num, df_denom, alpha, ncc=1):
+        '''Calculate the power of a F-test.
+
+        The effect size is Cohen's ``f^2``.
+
+        The sample size is given by ``nobs = df_denom + df_num + ncc``
+
+        Parameters
+        ----------
+        effect_size : float
+            The effect size is here Cohen's ``f2``. This is equal to
+            the noncentrality of an F-test divided by nobs.
+        df_num : int or float
+            Numerator degrees of freedom,
+            This corresponds to the number of constraints in Wald tests.
+        df_denom : int or float
+            Denominator degrees of freedom.
+            This corresponds to the df_resid in Wald tests.
+        alpha : float in interval (0,1)
+            Significance level, e.g. 0.05, is the probability of a type I
+            error, that is wrong rejections if the Null Hypothesis is true.
+        ncc : int
+            Degrees of freedom correction for non-centrality parameter.
+            see Notes
+
+        Returns
+        -------
+        power : float
+            Power of the test, e.g. 0.8, is one minus the probability of a
+            type II error. Power is the probability that the test correctly
+            rejects the Null Hypothesis if the Alternative Hypothesis is true.
+
+        Notes
+        -----
+        The sample size is given implicitly by df_denom
+
+        set ncc=0 to match t-test, or f-test in LikelihoodModelResults.
+        ncc=1 matches the non-centrality parameter in R::pwr::pwr.f2.test
+
+        ftest_power with ncc=0 should also be correct for f_test in regression
+        models, with df_num and d_denom as defined there. (not verified yet)
+        '''
+
+        pow_ = ftest_power_f2(effect_size, df_num, df_denom, alpha, ncc=ncc)
+        return pow_
+
+    #method is only added to have explicit keywords and docstring
+    def solve_power(self, effect_size=None, df_num=None, df_denom=None,
+                    alpha=None, power=None, ncc=1):
+        '''Solve for any one parameter of the power of a F-test
+
+        for the one sample F-test the keywords are:
+            effect_size, df_num, df_denom, alpha, power
+
+        Exactly one needs to be ``None``, all others need numeric values.
+
+        The effect size is Cohen's ``f2``.
+
+        The sample size is given by ``nobs = df_denom + df_num + ncc``, and
+        can be found by solving for df_denom.
+
+        Parameters
+        ----------
+        effect_size : float
+            The effect size is here Cohen's ``f2``. This is equal to
+            the noncentrality of an F-test divided by nobs.
+        df_num : int or float
+            Numerator degrees of freedom,
+            This corresponds to the number of constraints in Wald tests.
+        df_denom : int or float
+            Denominator degrees of freedom.
+            This corresponds to the df_resid in Wald tests.
+        alpha : float in interval (0,1)
+            significance level, e.g. 0.05, is the probability of a type I
+            error, that is wrong rejections if the Null Hypothesis is true.
+        power : float in interval (0,1)
+            power of the test, e.g. 0.8, is one minus the probability of a
+            type II error. Power is the probability that the test correctly
+            rejects the Null Hypothesis if the Alternative Hypothesis is true.
+        ncc : int
+            degrees of freedom correction for non-centrality parameter.
+            see Notes
+
+        Returns
+        -------
+        value : float
+            The value of the parameter that was set to None in the call. The
+            value solves the power equation given the remaining parameters.
 
 
         Notes
@@ -813,16 +1270,24 @@ class FTestPower(Power):
         where this fails.
 
         '''
-        return super(FTestPower, self).solve_power(effect_size=effect_size,
+
+        return super(FTestPowerF2, self).solve_power(effect_size=effect_size,
                                                       df_num=df_num,
                                                       df_denom=df_denom,
                                                       alpha=alpha,
                                                       power=power,
                                                       ncc=ncc)
 
+
 class FTestAnovaPower(Power):
     '''Statistical Power calculations F-test for one factor balanced ANOVA
 
+    This is based on Cohen's f as effect size measure.
+
+    See Also
+    --------
+    statsmodels.stats.oneway.effectsize_oneway
+
     '''
 
     def power(self, effect_size, nobs, alpha, k_groups=2):
@@ -831,8 +1296,8 @@ class FTestAnovaPower(Power):
         Parameters
         ----------
         effect_size : float
-            standardized effect size, mean divided by the standard deviation.
-            effect size has to be positive.
+            standardized effect size. The effect size is here Cohen's f, square
+            root of "f2".
         nobs : int or float
             sample size, number of observations.
         alpha : float in interval (0,1)
@@ -881,7 +1346,7 @@ class FTestAnovaPower(Power):
         -------
         value : float
             The value of the parameter that was set to None in the call. The
-            value solves the power equation given the remainding parameters.
+            value solves the power equation given the remaining parameters.
 
 
         Notes
@@ -895,7 +1360,7 @@ class FTestAnovaPower(Power):
 
         '''
         # update start values for root finding
-        if not k_groups is None:
+        if k_groups is not None:
             self.start_ttp['nobs'] = k_groups * 10
             self.start_bqexp['nobs'] = dict(low=k_groups * 2,
                                             start_upp=k_groups * 10)
@@ -936,8 +1401,7 @@ class GofChisquarePower(Power):
 
     '''
 
-    def power(self, effect_size, nobs, alpha, n_bins, ddof=0):
-              #alternative='two-sided'):
+    def power(self, effect_size, nobs, alpha, n_bins, ddof=0):#alternative='two-sided'):
         '''Calculate the power of a chisquare test for one sample
 
         Only two-sided alternative is implemented
@@ -1050,7 +1514,7 @@ class _GofChisquareIndPower(Power):
             sample 1. see description of nobs1
             The default for ratio is 1; to solve for ration given the other
             arguments it has to be explicitely set to None.
-        alternative : string, 'two-sided' (default) or 'one-sided'
+        alternative : str, 'two-sided' (default) or 'one-sided'
             extra argument to choose whether the power is calculated for a
             two-sided (default) or one sided test.
             'one-sided' assumes we are in the relevant tail.
@@ -1101,7 +1565,7 @@ class _GofChisquareIndPower(Power):
             sample 1. see description of nobs1
             The default for ratio is 1; to solve for ration given the other
             arguments it has to be explicitely set to None.
-        alternative : string, 'two-sided' (default) or 'one-sided'
+        alternative : str, 'two-sided' (default) or 'one-sided'
             extra argument to choose whether the power is calculated for a
             two-sided (default) or one sided test.
             'one-sided' assumes we are in the relevant tail.
@@ -1110,7 +1574,7 @@ class _GofChisquareIndPower(Power):
         -------
         value : float
             The value of the parameter that was set to None in the call. The
-            value solves the power equation given the remainding parameters.
+            value solves the power equation given the remaining parameters.
 
 
         Notes
